import {
	ArcElement,
	BarController,
	BarElement,
	CategoryScale,
	Chart,
	Colors,
	Legend,
	LinearScale,
	LineController,
	LineElement,
	PieController,
	PointElement,
	Tooltip,
	type ChartConfiguration,
	type ChartConfigurationCustomTypesPerDataset,
	type ChartType,
	type DefaultDataPoint,
} from "chart.js";
import { debounce } from "obsidian";

Chart.register(
	Colors,
	BarController,
	BarElement,
	CategoryScale,
	LinearScale,
	Legend,
	Tooltip,
	PieController,
	ArcElement,
	LineController,
	LineElement,
	PointElement
);

export function chart<
	Type extends ChartType = ChartType,
	Data = DefaultDataPoint<Type>,
	Label = unknown
>(
	canvas: HTMLCanvasElement,
	config:
		| ChartConfiguration<Type, Data, Label>
		| ChartConfigurationCustomTypesPerDataset<Type, Data, Label>
) {
	const chart = new Chart<Type, Data, Label>(canvas, config);

	const resizeObserver = new ResizeObserver(
		debounce(() => {
			requestAnimationFrame(() => {
				chart.resize();
			});
		}, 500)
	);

	resizeObserver.observe(canvas);

	return {
		update: (
			config:
				| ChartConfiguration<Type, Data, Label>
				| ChartConfigurationCustomTypesPerDataset<Type, Data, Label>
		) => {
			chart.data = config.data;
			if (config.options) {
				chart.options = config.options;
			}
			chart.update();
		},
		destroy: () => {
			resizeObserver.disconnect();
			chart.destroy();
		},
	};
}
